@@ -15,16 +15,12 @@
  */
 
 #include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
 #include <zend_smart_str.h>
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
+#include "phongo_error.h"
 
 zend_class_entry* php_phongo_maxkey_ce;
 
@@ -94,11 +90,39 @@ static PHP_METHOD(MaxKey, unserialize)
 	zend_restore_error_handling(&error_handling);
 } /* }}} */
 
+/* {{{ proto array MongoDB\Driver\MaxKey::__serialize()
+*/
+static PHP_METHOD(MaxKey, __serialize)
+{
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	array_init_size(return_value, 0);
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\MaxKey::__unserialize(array $data)
+*/
+static PHP_METHOD(MaxKey, __unserialize)
+{
+	zval* data;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(data)
+	PHONGO_PARSE_PARAMETERS_END();
+} /* }}} */
+
 /* {{{ MongoDB\BSON\MaxKey function entries */
+/* clang-format off */
 ZEND_BEGIN_ARG_INFO_EX(ai_MaxKey___set_state, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, properties, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_MaxKey___unserialize, 0, 0, 1)
+	ZEND_ARG_ARRAY_INFO(0, data, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_WITH_TENTATIVE_RETURN_TYPE_INFO_EX(ai_MaxKey_jsonSerialize, 0, 0, IS_ARRAY, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_MaxKey_unserialize, 0, 0, 1)
 	ZEND_ARG_INFO(0, serialized)
 ZEND_END_ARG_INFO()
@@ -107,14 +131,15 @@ ZEND_BEGIN_ARG_INFO_EX(ai_MaxKey_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_maxkey_me[] = {
-	/* clang-format off */
+	PHP_ME(MaxKey, __serialize, ai_MaxKey_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(MaxKey, __set_state, ai_MaxKey___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(MaxKey, jsonSerialize, ai_MaxKey_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(MaxKey, __unserialize, ai_MaxKey___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(MaxKey, jsonSerialize, ai_MaxKey_jsonSerialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(MaxKey, serialize, ai_MaxKey_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(MaxKey, unserialize, ai_MaxKey_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
-	/* clang-format on */
 };
+/* clang-format on */
 /* }}} */
 
 /* {{{ MongoDB\BSON\MaxKey object handlers */
@@ -129,9 +154,8 @@ static void php_phongo_maxkey_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_maxkey_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_maxkey_t* intern = NULL;
+	php_phongo_maxkey_t* intern = zend_object_alloc(sizeof(php_phongo_maxkey_t), class_type);
 
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_maxkey_t, class_type);
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
 
@@ -161,12 +185,3 @@ void php_phongo_maxkey_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_maxkey.free_obj  = php_phongo_maxkey_free_object;
 	php_phongo_handler_maxkey.offset    = XtOffsetOf(php_phongo_maxkey_t, std);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
