@@ -286,7 +286,7 @@ static bool php_phongo_bulkwrite_update_apply_options(bson_t* boptions, zval* zo
 	return true;
 } /* }}} */
 
-/* Applies options (including defaults) for an delete operation. */
+/* Applies options (including defaults) for a delete operation. */
 static bool php_phongo_bulkwrite_delete_apply_options(bson_t* boptions, zval* zoptions) /* {{{ */
 {
 	int32_t limit = 0;
@@ -334,6 +334,7 @@ static PHP_METHOD(BulkWrite, __construct)
 	intern->bulk     = mongoc_bulk_operation_new(ordered);
 	intern->ordered  = ordered;
 	intern->bypass   = PHONGO_BULKWRITE_BYPASS_UNSET;
+	intern->let      = NULL;
 	intern->num_ops  = 0;
 	intern->executed = false;
 
@@ -342,6 +343,38 @@ static PHP_METHOD(BulkWrite, __construct)
 		mongoc_bulk_operation_set_bypass_document_validation(intern->bulk, bypass);
 		intern->bypass = bypass;
 	}
+
+	if (options && php_array_existsc(options, "let")) {
+		zval* value = php_array_fetch(options, "let");
+
+		if (Z_TYPE_P(value) != IS_OBJECT && Z_TYPE_P(value) != IS_ARRAY) {
+			phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT, "Expected \"let\" option to be array or object, %s given", zend_get_type_by_const(Z_TYPE_P(value)));
+			return;
+		}
+
+		intern->let = bson_new();
+		php_phongo_zval_to_bson(value, PHONGO_BSON_NONE, intern->let, NULL);
+
+		if (EG(exception)) {
+			return;
+		}
+
+		mongoc_bulk_operation_set_let(intern->bulk, intern->let);
+	}
+
+	if (options && php_array_existsc(options, "comment")) {
+		zval* value = php_array_fetch(options, "comment");
+
+		intern->comment = ecalloc(1, sizeof(bson_value_t));
+		php_phongo_zval_to_bson_value(value, PHONGO_BSON_NONE, intern->comment);
+
+		if (EG(exception)) {
+			/* Exception should already have been thrown */
+			return;
+		}
+
+		mongoc_bulk_operation_set_comment(intern->bulk, intern->comment);
+	}
 } /* }}} */
 
 /* {{{ proto mixed MongoDB\Driver\BulkWrite::insert(array|object $document)
@@ -579,6 +612,15 @@ static void php_phongo_bulkwrite_free_object(zend_object* object) /* {{{ */
 		mongoc_bulk_operation_destroy(intern->bulk);
 	}
 
+	if (intern->let) {
+		bson_clear(&intern->let);
+	}
+
+	if (intern->comment) {
+		bson_value_destroy(intern->comment);
+		efree(intern->comment);
+	}
+
 	if (intern->database) {
 		efree(intern->database);
 	}
@@ -633,6 +675,28 @@ static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handl
 		ADD_ASSOC_NULL_EX(&retval, "bypassDocumentValidation");
 	}
 
+	if (intern->comment) {
+		zval zv;
+
+		if (!php_phongo_bson_value_to_zval(intern->comment, &zv)) {
+			zval_ptr_dtor(&zv);
+			goto done;
+		}
+
+		ADD_ASSOC_ZVAL_EX(&retval, "comment", &zv);
+	}
+
+	if (intern->let) {
+		zval zv;
+
+		if (!php_phongo_bson_to_zval(bson_get_data(intern->let), intern->let->len, &zv)) {
+			zval_ptr_dtor(&zv);
+			goto done;
+		}
+
+		ADD_ASSOC_ZVAL_EX(&retval, "let", &zv);
+	}
+
 	ADD_ASSOC_BOOL_EX(&retval, "executed", intern->executed);
 	ADD_ASSOC_LONG_EX(&retval, "server_id", mongoc_bulk_operation_get_hint(intern->bulk));
 
@@ -652,6 +716,7 @@ static HashTable* php_phongo_bulkwrite_get_debug_info(phongo_compat_object_handl
 		ADD_ASSOC_NULL_EX(&retval, "write_concern");
 	}
 
+done:
 	return Z_ARRVAL(retval);
 } /* }}} */
 /* }}} */
