@@ -15,16 +15,13 @@
  */
 
 #include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
 #include <zend_smart_str.h>
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
+#include "phongo_error.h"
+#include "phongo_util.h"
 
 zend_class_entry* php_phongo_cursorid_ce;
 
@@ -58,14 +55,14 @@ static bool php_phongo_cursorid_init_from_hash(php_phongo_cursorid_t* intern, Ha
 	return false;
 } /* }}} */
 
-static HashTable* php_phongo_cursorid_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
+static HashTable* php_phongo_cursorid_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp, bool is_serialize) /* {{{ */
 {
 	php_phongo_cursorid_t* intern;
 	HashTable*             props;
 
 	intern = Z_OBJ_CURSORID(PHONGO_COMPAT_GET_OBJ(object));
 
-	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_temp, intern, props, 1);
 
 	if (!intern->initialized) {
 		return props;
@@ -74,14 +71,18 @@ static HashTable* php_phongo_cursorid_get_properties_hash(phongo_compat_object_h
 	{
 		zval value;
 
-		if (is_debug) {
-#if SIZEOF_ZEND_LONG == 4
+		if (is_serialize) {
 			ZVAL_INT64_STRING(&value, intern->id);
+		} else {
+#if SIZEOF_ZEND_LONG == 4
+			if (intern->id > INT32_MAX || intern->id < INT32_MIN) {
+				ZVAL_INT64_STRING(&value, intern->id);
+			} else {
+				ZVAL_LONG(&value, intern->id);
+			}
 #else
 			ZVAL_LONG(&value, intern->id);
 #endif
-		} else {
-			ZVAL_INT64_STRING(&value, intern->id);
 		}
 		zend_hash_str_update(props, "id", sizeof("id") - 1, &value);
 	}
@@ -203,11 +204,41 @@ static PHP_METHOD(CursorId, unserialize)
 	zval_ptr_dtor(&props);
 } /* }}} */
 
+/* {{{ proto array MongoDB\Driver\CursorId::__serialize()
+*/
+static PHP_METHOD(CursorId, __serialize)
+{
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETURN_ARR(php_phongo_cursorid_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true, true));
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\CursorId::__unserialize(array $data)
+*/
+static PHP_METHOD(CursorId, __unserialize)
+{
+	zval* data;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(data)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	php_phongo_cursorid_init_from_hash(Z_CURSORID_OBJ_P(getThis()), Z_ARRVAL_P(data));
+} /* }}} */
+
 /* {{{ MongoDB\Driver\CursorId function entries */
+/* clang-format off */
 ZEND_BEGIN_ARG_INFO_EX(ai_CursorId___set_state, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, properties, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(ai_CursorId___toString, 0, 0, IS_STRING, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(ai_CursorId___unserialize, 0, 0, 1)
+	ZEND_ARG_ARRAY_INFO(0, data, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_CursorId_unserialize, 0, 0, 1)
 	ZEND_ARG_INFO(0, serialized)
 ZEND_END_ARG_INFO()
@@ -216,16 +247,16 @@ ZEND_BEGIN_ARG_INFO_EX(ai_CursorId_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
 
 static zend_function_entry php_phongo_cursorid_me[] = {
-	/* clang-format off */
+	PHP_ME(CursorId, __serialize, ai_CursorId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(CursorId, __set_state, ai_CursorId___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
-	PHP_ME(CursorId, __toString, ai_CursorId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(CursorId, __toString, ai_CursorId___toString, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(CursorId, __unserialize, ai_CursorId___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(CursorId, serialize, ai_CursorId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(CursorId, unserialize, ai_CursorId_unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	ZEND_NAMED_ME(__construct, PHP_FN(MongoDB_disabled___construct), ai_CursorId_void, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
-	ZEND_NAMED_ME(__wakeup, PHP_FN(MongoDB_disabled___wakeup), ai_CursorId_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_FE_END
-	/* clang-format on */
 };
+/* clang-format on */
 /* }}} */
 
 /* {{{ MongoDB\Driver\CursorId object handlers */
@@ -245,9 +276,7 @@ static void php_phongo_cursorid_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_cursorid_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_cursorid_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_cursorid_t, class_type);
+	php_phongo_cursorid_t* intern = zend_object_alloc(sizeof(php_phongo_cursorid_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -260,13 +289,13 @@ static zend_object* php_phongo_cursorid_create_object(zend_class_entry* class_ty
 static HashTable* php_phongo_cursorid_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
-	return php_phongo_cursorid_get_properties_hash(object, true);
+	return php_phongo_cursorid_get_properties_hash(object, true, false);
 } /* }}} */
 /* }}} */
 
 static HashTable* php_phongo_cursorid_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
-	return php_phongo_cursorid_get_properties_hash(object, false);
+	return php_phongo_cursorid_get_properties_hash(object, false, false);
 } /* }}} */
 
 void php_phongo_cursorid_init_ce(INIT_FUNC_ARGS) /* {{{ */
@@ -280,18 +309,13 @@ void php_phongo_cursorid_init_ce(INIT_FUNC_ARGS) /* {{{ */
 
 	zend_class_implements(php_phongo_cursorid_ce, 1, zend_ce_serializable);
 
+#if PHP_VERSION_ID >= 80000
+	zend_class_implements(php_phongo_cursorid_ce, 1, zend_ce_stringable);
+#endif
+
 	memcpy(&php_phongo_handler_cursorid, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_cursorid.get_debug_info = php_phongo_cursorid_get_debug_info;
 	php_phongo_handler_cursorid.get_properties = php_phongo_cursorid_get_properties;
 	php_phongo_handler_cursorid.free_obj       = php_phongo_cursorid_free_object;
 	php_phongo_handler_cursorid.offset         = XtOffsetOf(php_phongo_cursorid_t, std);
 } /* }}} */
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
