@@ -311,18 +311,16 @@ static PHP_METHOD(Manager, addSubscriber)
    Return a ClientEncryption instance */
 static PHP_METHOD(Manager, createClientEncryption)
 {
-	zend_error_handling error_handling;
-	zval*               options;
+	zval* options;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "a", &options) == FAILURE) {
-		zend_restore_error_handling(&error_handling);
-		return;
-	}
-	zend_restore_error_handling(&error_handling);
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(options)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	object_init_ex(return_value, php_phongo_clientencryption_ce);
 
 	/* An exception will be thrown on error. */
-	phongo_clientencryption_init(return_value, getThis(), options);
+	phongo_clientencryption_init(Z_CLIENTENCRYPTION_OBJ_P(return_value), options, getThis());
 } /* }}} */
 
 /* {{{ proto MongoDB\Driver\Cursor MongoDB\Driver\Manager::executeCommand(string $db, MongoDB\Driver\Command $command[, array $options = null])
@@ -608,6 +606,21 @@ cleanup:
 	}
 } /* }}} */
 
+/* {{{ proto array|object|null MongoDB\Driver\Manager::getEncryptedFieldsMap()
+   Returns the autoEncryption.encryptedFieldsMap driver option */
+static PHP_METHOD(Manager, getEncryptedFieldsMap)
+{
+	php_phongo_manager_t* intern;
+
+	intern = Z_MANAGER_OBJ_P(getThis());
+
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	if (!Z_ISUNDEF(intern->enc_fields_map)) {
+		RETURN_ZVAL(&intern->enc_fields_map, 1, 0);
+	}
+} /* }}} */
+
 /* {{{ proto MongoDB\Driver\ReadConcern MongoDB\Driver\Manager::getReadConcern()
    Returns the ReadConcern associated with this Manager */
 static PHP_METHOD(Manager, getReadConcern)
@@ -894,6 +907,7 @@ static zend_function_entry php_phongo_manager_me[] = {
 	PHP_ME(Manager, executeReadWriteCommand, ai_Manager_executeCommand, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, executeQuery, ai_Manager_executeQuery, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, executeBulkWrite, ai_Manager_executeBulkWrite, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Manager, getEncryptedFieldsMap, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, getReadConcern, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, getReadPreference, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Manager, getServers, ai_Manager_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -938,6 +952,10 @@ static void php_phongo_manager_free_object(zend_object* object) /* {{{ */
 		efree(intern->client_hash);
 	}
 
+	if (!Z_ISUNDEF(intern->enc_fields_map)) {
+		zval_ptr_dtor(&intern->enc_fields_map);
+	}
+
 	/* Free the keyVaultClient last to ensure that potential non-persistent
 	 * clients are destroyed in the correct order */
 	if (!Z_ISUNDEF(intern->key_vault_client_manager)) {
