@@ -14,19 +14,21 @@
  * limitations under the License.
  */
 
-#include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
-#include <zend_smart_str.h>
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <php.h>
+#include <zend_smart_str.h>
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
 #include "php_array_api.h"
-#include "phongo_compat.h"
+
 #include "php_phongo.h"
-#include "php_bson.h"
+#include "phongo_bson_encode.h"
+#include "phongo_error.h"
+
+#include "MongoDB/ReadPreference.h"
 
 zend_class_entry* php_phongo_readpreference_ce;
 
@@ -469,7 +471,7 @@ static PHP_METHOD(ReadPreference, getTagSets)
 	}
 } /* }}} */
 
-static HashTable* php_phongo_readpreference_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug) /* {{{ */
+static HashTable* php_phongo_readpreference_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp) /* {{{ */
 {
 	php_phongo_readpreference_t* intern;
 	HashTable*                   props;
@@ -480,7 +482,7 @@ static HashTable* php_phongo_readpreference_get_properties_hash(phongo_compat_ob
 
 	intern = Z_OBJ_READPREFERENCE(PHONGO_COMPAT_GET_OBJ(object));
 
-	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 4);
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_temp, intern, props, 4);
 
 	if (!intern->read_preference) {
 		return props;
@@ -515,6 +517,8 @@ static HashTable* php_phongo_readpreference_get_properties_hash(phongo_compat_ob
 	}
 
 	if (mongoc_read_prefs_get_max_staleness_seconds(intern->read_preference) != MONGOC_NO_MAX_STALENESS) {
+		/* Note: valid values for maxStalesnessSeconds will not exceed the range
+		 * of 32-bit signed integers, so conditional encoding is not necessary. */
 		long maxStalenessSeconds = mongoc_read_prefs_get_max_staleness_seconds(intern->read_preference);
 		zval z_max_ss;
 
@@ -675,6 +679,28 @@ static PHP_METHOD(ReadPreference, unserialize)
 	zval_ptr_dtor(&props);
 } /* }}} */
 
+/* {{{ proto array MongoDB\Driver\ReadPreference::__serialize()
+*/
+static PHP_METHOD(ReadPreference, __serialize)
+{
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETURN_ARR(php_phongo_readpreference_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true));
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\ReadPreference::__unserialize(array $data)
+*/
+static PHP_METHOD(ReadPreference, __unserialize)
+{
+	zval* data;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(data)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	php_phongo_readpreference_init_from_hash(Z_READPREFERENCE_OBJ_P(getThis()), Z_ARRVAL_P(data));
+} /* }}} */
+
 /* {{{ MongoDB\Driver\ReadPreference function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_ReadPreference___construct, 0, 0, 1)
 	ZEND_ARG_INFO(0, mode)
@@ -686,6 +712,10 @@ ZEND_BEGIN_ARG_INFO_EX(ai_ReadPreference___set_state, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, properties, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_ReadPreference___unserialize, 0, 0, 1)
+	ZEND_ARG_ARRAY_INFO(0, data, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_ReadPreference_unserialize, 0, 0, 1)
 	ZEND_ARG_INFO(0, serialized)
 ZEND_END_ARG_INFO()
@@ -696,7 +726,9 @@ ZEND_END_ARG_INFO()
 static zend_function_entry php_phongo_readpreference_me[] = {
 	/* clang-format off */
 	PHP_ME(ReadPreference, __construct, ai_ReadPreference___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(ReadPreference, __serialize, ai_ReadPreference_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ReadPreference, __set_state, ai_ReadPreference___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+	PHP_ME(ReadPreference, __unserialize, ai_ReadPreference___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ReadPreference, getHedge, ai_ReadPreference_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ReadPreference, getMaxStalenessSeconds, ai_ReadPreference_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(ReadPreference, getMode, ai_ReadPreference_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -731,9 +763,7 @@ static void php_phongo_readpreference_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_readpreference_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_readpreference_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_readpreference_t, class_type);
+	php_phongo_readpreference_t* intern = zend_object_alloc(sizeof(php_phongo_readpreference_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -788,11 +818,77 @@ void php_phongo_readpreference_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_declare_class_constant_string(php_phongo_readpreference_ce, ZEND_STRL("NEAREST"), PHONGO_READ_NEAREST);
 } /* }}} */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+void phongo_readpreference_init(zval* return_value, const mongoc_read_prefs_t* read_prefs) /* {{{ */
+{
+	php_phongo_readpreference_t* intern;
+
+	object_init_ex(return_value, php_phongo_readpreference_ce);
+
+	intern                  = Z_READPREFERENCE_OBJ_P(return_value);
+	intern->read_preference = mongoc_read_prefs_copy(read_prefs);
+}
+/* }}} */
+
+const mongoc_read_prefs_t* phongo_read_preference_from_zval(zval* zread_preference) /* {{{ */
+{
+	if (zread_preference) {
+		php_phongo_readpreference_t* intern = Z_READPREFERENCE_OBJ_P(zread_preference);
+
+		if (intern) {
+			return intern->read_preference;
+		}
+	}
+
+	return NULL;
+} /* }}} */
+/* }}} */
+
+/* Prepare tagSets for BSON encoding by converting each array in the set to an
+ * object. This ensures that empty arrays will serialize as empty documents.
+ *
+ * php_phongo_read_preference_tags_are_valid() handles actual validation of the
+ * tag set structure. */
+void php_phongo_read_preference_prep_tagsets(zval* tagSets) /* {{{ */
+{
+	HashTable* ht_data;
+	zval*      tagSet;
+
+	if (Z_TYPE_P(tagSets) != IS_ARRAY) {
+		return;
+	}
+
+	ht_data = HASH_OF(tagSets);
+
+	ZEND_HASH_FOREACH_VAL_IND(ht_data, tagSet)
+	{
+		ZVAL_DEREF(tagSet);
+		if (Z_TYPE_P(tagSet) == IS_ARRAY) {
+			SEPARATE_ZVAL_NOREF(tagSet);
+			convert_to_object(tagSet);
+		}
+	}
+	ZEND_HASH_FOREACH_END();
+} /* }}} */
+
+/* Checks if tags is valid to set on a mongoc_read_prefs_t. It may be null or an
+ * array of one or more documents. */
+bool php_phongo_read_preference_tags_are_valid(const bson_t* tags) /* {{{ */
+{
+	bson_iter_t iter;
+
+	if (bson_empty0(tags)) {
+		return true;
+	}
+
+	if (!bson_iter_init(&iter, tags)) {
+		return false;
+	}
+
+	while (bson_iter_next(&iter)) {
+		if (!BSON_ITER_HOLDS_DOCUMENT(&iter)) {
+			return false;
+		}
+	}
+
+	return true;
+} /* }}} */
