@@ -14,16 +14,21 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
+
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
-#include "php_bson.h"
+#include "phongo_bson_encode.h"
+#include "phongo_client.h"
+#include "phongo_error.h"
+#include "phongo_execute.h"
+#include "phongo_util.h"
+
+#include "MongoDB/Server.h"
+#include "MongoDB/ServerDescription.h"
 
 zend_class_entry* php_phongo_server_ce;
 
@@ -295,67 +300,86 @@ static PHP_METHOD(Server, getTags)
 } /* }}} */
 
 /* {{{ proto array MongoDB\Driver\Server::getInfo()
-   Returns the last hello response for this Server */
+   Returns the last hello response for this Server or, in the case of a load
+   balancer, the initial handshake response. */
 static PHP_METHOD(Server, getInfo)
 {
-	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
+	mongoc_client_t*             client;
 	mongoc_server_description_t* sd;
+	mongoc_server_description_t* handshake_sd = NULL;
+	const bson_t*                hello_response;
+	php_phongo_bson_state        state;
+
+	PHONGO_PARSE_PARAMETERS_NONE();
 
 	intern = Z_SERVER_OBJ_P(getThis());
+	client = Z_MANAGER_OBJ_P(&intern->manager)->client;
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
+	if (!(sd = mongoc_client_get_server_description(client, intern->server_id))) {
+		phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get server description");
 		return;
 	}
-	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
-		const bson_t*         hello_response = mongoc_server_description_hello_response(sd);
-		php_phongo_bson_state state;
+	hello_response = mongoc_server_description_hello_response(sd);
 
-		PHONGO_BSON_INIT_DEBUG_STATE(state);
+	/* If the server description is a load balancer, its hello_response will be
+	 * empty. Instead, report the hello_response from the handshake description
+	 * (i.e. backing server). */
+	if (!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_LOAD_BALANCER].name)) {
+		bson_error_t error = { 0 };
 
-		if (!php_phongo_bson_to_zval_ex(bson_get_data(hello_response), hello_response->len, &state)) {
-			/* Exception should already have been thrown */
-			zval_ptr_dtor(&state.zchild);
-			mongoc_server_description_destroy(sd);
-			return;
+		if (!(handshake_sd = mongoc_client_get_handshake_description(client, intern->server_id, NULL, &error))) {
+			phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get handshake server description: %s", error.message);
+			goto cleanup;
 		}
 
-		mongoc_server_description_destroy(sd);
-
-		RETURN_ZVAL(&state.zchild, 0, 1);
+		hello_response = mongoc_server_description_hello_response(handshake_sd);
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get server description");
+	PHONGO_BSON_INIT_DEBUG_STATE(state);
+
+	if (!php_phongo_bson_to_zval_ex(bson_get_data(hello_response), hello_response->len, &state)) {
+		/* Exception should already have been thrown */
+		zval_ptr_dtor(&state.zchild);
+		goto cleanup;
+	}
+
+	RETVAL_ZVAL(&state.zchild, 0, 1);
+
+cleanup:
+	if (handshake_sd) {
+		mongoc_server_description_destroy(handshake_sd);
+	}
+
+	mongoc_server_description_destroy(sd);
 } /* }}} */
 
-/* {{{ proto integer MongoDB\Driver\Server::getLatency()
-   Returns the last measured latency for this Server */
+/* {{{ proto integer|null MongoDB\Driver\Server::getLatency()
+   Returns the measured latency (i.e. round trip time in milliseconds) for
+   this Server, or null if unset. */
 static PHP_METHOD(Server, getLatency)
 {
-	zend_error_handling          error_handling;
 	php_phongo_server_t*         intern;
 	mongoc_server_description_t* sd;
 
+	PHONGO_PARSE_PARAMETERS_NONE();
+
 	intern = Z_SERVER_OBJ_P(getThis());
 
-	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
-	if (zend_parse_parameters_none() == FAILURE) {
-		zend_restore_error_handling(&error_handling);
+	if (!(sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
+		phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get server description");
 		return;
 	}
-	zend_restore_error_handling(&error_handling);
 
-	if ((sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
+	/* TODO: Use MONGOC_RTT_UNSET once it is added to libmongoc's public API (CDRIVER-4176) */
+	if (mongoc_server_description_round_trip_time(sd) == -1) {
+		RETVAL_NULL();
+	} else {
 		RETVAL_LONG((zend_long) mongoc_server_description_round_trip_time(sd));
-		mongoc_server_description_destroy(sd);
-		return;
 	}
 
-	phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get server description");
+	mongoc_server_description_destroy(sd);
 } /* }}} */
 
 /* {{{ proto integer MongoDB\Driver\Server::getPort()
@@ -384,6 +408,21 @@ static PHP_METHOD(Server, getPort)
 	phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get server description");
 } /* }}} */
 
+/* {{{ proto MongoDB\Driver\ServerDescription MongoDB\Driver\Server::getServerDescription()
+   Returns the server description for this Server */
+static PHP_METHOD(Server, getServerDescription)
+{
+	mongoc_server_description_t* server_description;
+	php_phongo_server_t*         intern = Z_SERVER_OBJ_P(getThis());
+
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	server_description = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id);
+
+	/* Avoid making another copy in phongo_serverdescription_init */
+	phongo_serverdescription_init_ex(return_value, server_description, false);
+} /* }}} */
+
 /* {{{ proto integer MongoDB\Driver\Server::getType()
    Returns the node type of this Server */
 static PHP_METHOD(Server, getType)
@@ -585,6 +624,7 @@ static zend_function_entry php_phongo_server_me[] = {
 	PHP_ME(Server, getInfo, ai_Server_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Server, getLatency, ai_Server_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Server, getPort, ai_Server_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(Server, getServerDescription, ai_Server_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Server, getType, ai_Server_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Server, isPrimary, ai_Server_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(Server, isSecondary, ai_Server_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -646,9 +686,7 @@ static void php_phongo_server_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_server_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_server_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_server_t, class_type);
+	php_phongo_server_t* intern = zend_object_alloc(sizeof(php_phongo_server_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -664,17 +702,19 @@ static HashTable* php_phongo_server_get_debug_info(phongo_compat_object_handler_
 {
 	php_phongo_server_t*         intern = NULL;
 	zval                         retval = ZVAL_STATIC_INIT;
+	mongoc_client_t*             client;
 	mongoc_server_description_t* sd;
 
 	*is_temp = 1;
 	intern   = Z_OBJ_SERVER(PHONGO_COMPAT_GET_OBJ(object));
+	client   = Z_MANAGER_OBJ_P(&intern->manager)->client;
 
-	if (!(sd = mongoc_client_get_server_description(Z_MANAGER_OBJ_P(&intern->manager)->client, intern->server_id))) {
+	if (!(sd = mongoc_client_get_server_description(client, intern->server_id))) {
 		phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get server description");
 		return NULL;
 	}
 
-	php_phongo_server_to_zval(&retval, sd);
+	php_phongo_server_to_zval(&retval, client, sd);
 	mongoc_server_description_destroy(sd);
 
 	return Z_ARRVAL(retval);
@@ -706,13 +746,101 @@ void php_phongo_server_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_ARBITER"), PHONGO_SERVER_RS_ARBITER);
 	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_OTHER"), PHONGO_SERVER_RS_OTHER);
 	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_RS_GHOST"), PHONGO_SERVER_RS_GHOST);
+	zend_declare_class_constant_long(php_phongo_server_ce, ZEND_STRL("TYPE_LOAD_BALANCER"), PHONGO_SERVER_LOAD_BALANCER);
 } /* }}} */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+void phongo_server_init(zval* return_value, zval* manager, uint32_t server_id) /* {{{ */
+{
+	php_phongo_server_t* server;
+
+	object_init_ex(return_value, php_phongo_server_ce);
+
+	server            = Z_SERVER_OBJ_P(return_value);
+	server->server_id = server_id;
+
+	ZVAL_ZVAL(&server->manager, manager, 1, 0);
+}
+/* }}} */
+
+bool php_phongo_server_to_zval(zval* retval, mongoc_client_t* client, mongoc_server_description_t* sd) /* {{{ */
+{
+	mongoc_host_list_t* host           = mongoc_server_description_host(sd);
+	const bson_t*       hello_response = mongoc_server_description_hello_response(sd);
+	bson_iter_t         iter;
+
+	array_init(retval);
+
+	ADD_ASSOC_STRING(retval, "host", host->host);
+	ADD_ASSOC_LONG_EX(retval, "port", host->port);
+	ADD_ASSOC_LONG_EX(retval, "type", php_phongo_server_description_type(sd));
+	ADD_ASSOC_BOOL_EX(retval, "is_primary", !strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_PRIMARY].name));
+	ADD_ASSOC_BOOL_EX(retval, "is_secondary", !strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_SECONDARY].name));
+	ADD_ASSOC_BOOL_EX(retval, "is_arbiter", !strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_RS_ARBITER].name));
+	ADD_ASSOC_BOOL_EX(retval, "is_hidden", bson_iter_init_find_case(&iter, hello_response, "hidden") && bson_iter_as_bool(&iter));
+	ADD_ASSOC_BOOL_EX(retval, "is_passive", bson_iter_init_find_case(&iter, hello_response, "passive") && bson_iter_as_bool(&iter));
+
+	if (bson_iter_init_find(&iter, hello_response, "tags") && BSON_ITER_HOLDS_DOCUMENT(&iter)) {
+		const uint8_t*        bytes;
+		uint32_t              len;
+		php_phongo_bson_state state;
+
+		PHONGO_BSON_INIT_DEBUG_STATE(state);
+		bson_iter_document(&iter, &len, &bytes);
+		if (!php_phongo_bson_to_zval_ex(bytes, len, &state)) {
+			/* Exception already thrown */
+			zval_ptr_dtor(&state.zchild);
+			return false;
+		}
+
+		ADD_ASSOC_ZVAL_EX(retval, "tags", &state.zchild);
+	}
+
+	/* If the server description is a load balancer, its hello_response will be
+	 * empty. Instead, report the hello_response from the handshake description
+	 * (i.e. backing server). */
+	if (!strcmp(mongoc_server_description_type(sd), php_phongo_server_description_type_map[PHONGO_SERVER_LOAD_BALANCER].name)) {
+		const bson_t*                handshake_response;
+		mongoc_server_description_t* handshake_sd;
+		bson_error_t                 error = { 0 };
+		php_phongo_bson_state        state;
+
+		if (!(handshake_sd = mongoc_client_get_handshake_description(client, mongoc_server_description_id(sd), NULL, &error))) {
+			phongo_throw_exception(PHONGO_ERROR_RUNTIME, "Failed to get handshake server description: %s", error.message);
+			return false;
+		}
+
+		PHONGO_BSON_INIT_DEBUG_STATE(state);
+		handshake_response = mongoc_server_description_hello_response(handshake_sd);
+
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(handshake_response), handshake_response->len, &state)) {
+			/* Exception already thrown */
+			mongoc_server_description_destroy(handshake_sd);
+			zval_ptr_dtor(&state.zchild);
+			return false;
+		}
+
+		ADD_ASSOC_ZVAL_EX(retval, "last_hello_response", &state.zchild);
+		mongoc_server_description_destroy(handshake_sd);
+	} else {
+		php_phongo_bson_state state;
+
+		PHONGO_BSON_INIT_DEBUG_STATE(state);
+
+		if (!php_phongo_bson_to_zval_ex(bson_get_data(hello_response), hello_response->len, &state)) {
+			/* Exception already thrown */
+			zval_ptr_dtor(&state.zchild);
+			return false;
+		}
+
+		ADD_ASSOC_ZVAL_EX(retval, "last_hello_response", &state.zchild);
+	}
+
+	/* TODO: Use MONGOC_RTT_UNSET once it is added to libmongoc's public API (CDRIVER-4176) */
+	if (mongoc_server_description_round_trip_time(sd) == -1) {
+		ADD_ASSOC_NULL_EX(retval, "round_trip_time");
+	} else {
+		ADD_ASSOC_LONG_EX(retval, "round_trip_time", (zend_long) mongoc_server_description_round_trip_time(sd));
+	}
+
+	return true;
+} /* }}} */
