@@ -14,17 +14,18 @@
  * limitations under the License.
  */
 
+#include "mongoc/mongoc.h"
+
 #include <php.h>
-#include <Zend/zend_interfaces.h>
-#include <ext/standard/php_var.h>
 #include <zend_smart_str.h>
+#include <ext/standard/php_var.h>
+#include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "phongo_compat.h"
 #include "php_phongo.h"
+#include "phongo_error.h"
+#include "phongo_util.h"
+
+#include "MongoDB/WriteConcern.h"
 
 zend_class_entry* php_phongo_writeconcern_ce;
 
@@ -298,7 +299,7 @@ static PHP_METHOD(WriteConcern, isDefault)
 	RETURN_BOOL(mongoc_write_concern_is_default(intern->write_concern));
 } /* }}} */
 
-static HashTable* php_phongo_write_concern_get_properties_hash(phongo_compat_object_handler_type* object, bool is_debug, bool is_bson) /* {{{ */
+static HashTable* php_phongo_writeconcern_get_properties_hash(phongo_compat_object_handler_type* object, bool is_temp, bool is_bson, bool is_serialize) /* {{{ */
 {
 	php_phongo_writeconcern_t* intern;
 	HashTable*                 props;
@@ -308,7 +309,7 @@ static HashTable* php_phongo_write_concern_get_properties_hash(phongo_compat_obj
 
 	intern = Z_OBJ_WRITECONCERN(PHONGO_COMPAT_GET_OBJ(object));
 
-	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 4);
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_temp, intern, props, 4);
 
 	if (!intern->write_concern) {
 		return props;
@@ -344,6 +345,12 @@ static HashTable* php_phongo_write_concern_get_properties_hash(phongo_compat_obj
 
 			if (is_bson) {
 				ZVAL_INT64(&z_wtimeout, wtimeout);
+			} else if (is_serialize) {
+				if (wtimeout > INT32_MAX || wtimeout < INT32_MIN) {
+					ZVAL_INT64_STRING(&z_wtimeout, wtimeout);
+				} else {
+					ZVAL_LONG(&z_wtimeout, wtimeout);
+				}
 			} else {
 #if SIZEOF_ZEND_LONG == 4
 				if (wtimeout > INT32_MAX || wtimeout < INT32_MIN) {
@@ -376,7 +383,7 @@ static PHP_METHOD(WriteConcern, bsonSerialize)
 	}
 	zend_restore_error_handling(&error_handling);
 
-	ZVAL_ARR(return_value, php_phongo_write_concern_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true, true));
+	ZVAL_ARR(return_value, php_phongo_writeconcern_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true, true, false));
 	convert_to_object(return_value);
 } /* }}} */
 
@@ -481,6 +488,28 @@ static PHP_METHOD(WriteConcern, unserialize)
 	zval_ptr_dtor(&props);
 } /* }}} */
 
+/* {{{ proto array MongoDB\Driver\WriteConcern::__serialize()
+*/
+static PHP_METHOD(WriteConcern, __serialize)
+{
+	PHONGO_PARSE_PARAMETERS_NONE();
+
+	RETURN_ARR(php_phongo_writeconcern_get_properties_hash(PHONGO_COMPAT_OBJ_P(getThis()), true, false, true));
+} /* }}} */
+
+/* {{{ proto void MongoDB\Driver\WriteConcern::__unserialize(array $data)
+*/
+static PHP_METHOD(WriteConcern, __unserialize)
+{
+	zval* data;
+
+	PHONGO_PARSE_PARAMETERS_START(1, 1)
+	Z_PARAM_ARRAY(data)
+	PHONGO_PARSE_PARAMETERS_END();
+
+	php_phongo_writeconcern_init_from_hash(Z_WRITECONCERN_OBJ_P(getThis()), Z_ARRVAL_P(data));
+} /* }}} */
+
 /* {{{ MongoDB\Driver\WriteConcern function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern___construct, 0, 0, 1)
 	ZEND_ARG_INFO(0, w)
@@ -492,6 +521,10 @@ ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern___set_state, 0, 0, 1)
 	ZEND_ARG_ARRAY_INFO(0, properties, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern___unserialize, 0, 0, 1)
+	ZEND_ARG_ARRAY_INFO(0, data, 0)
+ZEND_END_ARG_INFO()
+
 ZEND_BEGIN_ARG_INFO_EX(ai_WriteConcern_unserialize, 0, 0, 1)
 	ZEND_ARG_INFO(0, serialized)
 ZEND_END_ARG_INFO()
@@ -502,7 +535,9 @@ ZEND_END_ARG_INFO()
 static zend_function_entry php_phongo_writeconcern_me[] = {
 	/* clang-format off */
 	PHP_ME(WriteConcern, __construct, ai_WriteConcern___construct, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
+	PHP_ME(WriteConcern, __serialize, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(WriteConcern, __set_state, ai_WriteConcern___set_state, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
+	PHP_ME(WriteConcern, __unserialize, ai_WriteConcern___unserialize, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(WriteConcern, getW, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(WriteConcern, getWtimeout, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
 	PHP_ME(WriteConcern, getJournal, ai_WriteConcern_void, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
@@ -536,9 +571,7 @@ static void php_phongo_writeconcern_free_object(zend_object* object) /* {{{ */
 
 static zend_object* php_phongo_writeconcern_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_writeconcern_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_writeconcern_t, class_type);
+	php_phongo_writeconcern_t* intern = zend_object_alloc(sizeof(php_phongo_writeconcern_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -551,12 +584,12 @@ static zend_object* php_phongo_writeconcern_create_object(zend_class_entry* clas
 static HashTable* php_phongo_writeconcern_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	*is_temp = 1;
-	return php_phongo_write_concern_get_properties_hash(object, true, false);
+	return php_phongo_writeconcern_get_properties_hash(object, true, false, false);
 } /* }}} */
 
 static HashTable* php_phongo_writeconcern_get_properties(phongo_compat_object_handler_type* object) /* {{{ */
 {
-	return php_phongo_write_concern_get_properties_hash(object, false, false);
+	return php_phongo_writeconcern_get_properties_hash(object, false, false, false);
 } /* }}} */
 
 void php_phongo_writeconcern_init_ce(INIT_FUNC_ARGS) /* {{{ */
@@ -580,11 +613,59 @@ void php_phongo_writeconcern_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	zend_declare_class_constant_stringl(php_phongo_writeconcern_ce, ZEND_STRL("MAJORITY"), ZEND_STRL(PHONGO_WRITE_CONCERN_W_MAJORITY));
 } /* }}} */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+void phongo_writeconcern_init(zval* return_value, const mongoc_write_concern_t* write_concern) /* {{{ */
+{
+	php_phongo_writeconcern_t* intern;
+
+	object_init_ex(return_value, php_phongo_writeconcern_ce);
+
+	intern                = Z_WRITECONCERN_OBJ_P(return_value);
+	intern->write_concern = mongoc_write_concern_copy(write_concern);
+}
+/* }}} */
+
+const mongoc_write_concern_t* phongo_write_concern_from_zval(zval* zwrite_concern) /* {{{ */
+{
+	if (zwrite_concern) {
+		php_phongo_writeconcern_t* intern = Z_WRITECONCERN_OBJ_P(zwrite_concern);
+
+		if (intern) {
+			return intern->write_concern;
+		}
+	}
+
+	return NULL;
+} /* }}} */
+
+void php_phongo_write_concern_to_zval(zval* retval, const mongoc_write_concern_t* write_concern) /* {{{ */
+{
+	const char*   wtag     = mongoc_write_concern_get_wtag(write_concern);
+	const int32_t w        = mongoc_write_concern_get_w(write_concern);
+	const int64_t wtimeout = mongoc_write_concern_get_wtimeout_int64(write_concern);
+
+	array_init_size(retval, 4);
+
+	if (wtag) {
+		ADD_ASSOC_STRING(retval, "w", wtag);
+	} else if (mongoc_write_concern_get_wmajority(write_concern)) {
+		ADD_ASSOC_STRING(retval, "w", PHONGO_WRITE_CONCERN_W_MAJORITY);
+	} else if (w != MONGOC_WRITE_CONCERN_W_DEFAULT) {
+		ADD_ASSOC_LONG_EX(retval, "w", w);
+	}
+
+	if (mongoc_write_concern_journal_is_set(write_concern)) {
+		ADD_ASSOC_BOOL_EX(retval, "j", mongoc_write_concern_get_journal(write_concern));
+	}
+
+	if (wtimeout != 0) {
+#if SIZEOF_ZEND_LONG == 4
+		if (wtimeout > INT32_MAX || wtimeout < INT32_MIN) {
+			ADD_ASSOC_INT64_AS_STRING(retval, "wtimeout", wtimeout);
+		} else {
+			ADD_ASSOC_LONG_EX(retval, "wtimeout", wtimeout);
+		}
+#else
+		ADD_ASSOC_LONG_EX(retval, "wtimeout", wtimeout);
+#endif
+	}
+} /* }}} */
