@@ -1,5 +1,5 @@
 /*
- * Copyright 2014-2017 MongoDB, Inc.
+ * Copyright 2014-present MongoDB, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -14,17 +14,21 @@
  * limitations under the License.
  */
 
+#include "bson/bson.h"
+#include "mongoc/mongoc.h"
+
 #include <php.h>
 #include <Zend/zend_interfaces.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "php_array_api.h"
-#include "phongo_compat.h"
+
 #include "php_phongo.h"
-#include "php_bson.h"
+#include "phongo_error.h"
+
+#include "MongoDB/Server.h"
+#include "MongoDB/WriteConcern.h"
+#include "MongoDB/WriteConcernError.h"
+#include "MongoDB/WriteError.h"
 
 #define PHONGO_WRITERESULT_RETURN_LONG_FROM_BSON_INT32(iter, bson, key)                \
 	if (bson_iter_init_find((iter), (bson), (key)) && BSON_ITER_HOLDS_INT32((iter))) { \
@@ -109,14 +113,18 @@ static bool php_phongo_writeresult_get_writeerrors(php_phongo_writeresult_t* int
    Returns the number of documents that were inserted */
 static PHP_METHOD(WriteResult, getInsertedCount)
 {
+	zend_error_handling       error_handling;
 	bson_iter_t               iter;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	PHONGO_WRITERESULT_RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nInserted");
 } /* }}} */
@@ -125,14 +133,18 @@ static PHP_METHOD(WriteResult, getInsertedCount)
    Returns the number of documents that matched the update criteria */
 static PHP_METHOD(WriteResult, getMatchedCount)
 {
+	zend_error_handling       error_handling;
 	bson_iter_t               iter;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	PHONGO_WRITERESULT_RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nMatched");
 } /* }}} */
@@ -141,14 +153,18 @@ static PHP_METHOD(WriteResult, getMatchedCount)
    Returns the number of documents that were actually modified by an update */
 static PHP_METHOD(WriteResult, getModifiedCount)
 {
+	zend_error_handling       error_handling;
 	bson_iter_t               iter;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	PHONGO_WRITERESULT_RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nModified");
 } /* }}} */
@@ -157,14 +173,18 @@ static PHP_METHOD(WriteResult, getModifiedCount)
    Returns the number of documents that were deleted */
 static PHP_METHOD(WriteResult, getDeletedCount)
 {
+	zend_error_handling       error_handling;
 	bson_iter_t               iter;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	PHONGO_WRITERESULT_RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nRemoved");
 } /* }}} */
@@ -173,14 +193,18 @@ static PHP_METHOD(WriteResult, getDeletedCount)
    Returns the number of documents that were upserted */
 static PHP_METHOD(WriteResult, getUpsertedCount)
 {
+	zend_error_handling       error_handling;
 	bson_iter_t               iter;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	PHONGO_WRITERESULT_RETURN_LONG_FROM_BSON_INT32(&iter, intern->reply, "nUpserted");
 } /* }}} */
@@ -189,29 +213,37 @@ static PHP_METHOD(WriteResult, getUpsertedCount)
    Returns the Server from which the result originated */
 static PHP_METHOD(WriteResult, getServer)
 {
+	zend_error_handling       error_handling;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
-	phongo_server_init(return_value, intern->client, intern->server_id);
+	phongo_server_init(return_value, &intern->manager, intern->server_id);
 } /* }}} */
 
 /* {{{ proto array MongoDB\Driver\WriteResult::getUpsertedIds()
    Returns the identifiers generated by the server for upsert operations. */
 static PHP_METHOD(WriteResult, getUpsertedIds)
 {
+	zend_error_handling       error_handling;
 	bson_iter_t               iter, child;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	array_init(return_value);
 
@@ -247,13 +279,17 @@ static PHP_METHOD(WriteResult, getUpsertedIds)
    Return any write concern error that occurred */
 static PHP_METHOD(WriteResult, getWriteConcernError)
 {
+	zend_error_handling       error_handling;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	php_phongo_writeresult_get_writeconcernerror(intern, return_value);
 } /* }}} */
@@ -262,13 +298,17 @@ static PHP_METHOD(WriteResult, getWriteConcernError)
    Returns any write errors that occurred */
 static PHP_METHOD(WriteResult, getWriteErrors)
 {
+	zend_error_handling       error_handling;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	php_phongo_writeresult_get_writeerrors(intern, return_value);
 } /* }}} */
@@ -278,13 +318,17 @@ static PHP_METHOD(WriteResult, getWriteErrors)
    concern). */
 static PHP_METHOD(WriteResult, isAcknowledged)
 {
+	zend_error_handling       error_handling;
 	php_phongo_writeresult_t* intern;
 
 	intern = Z_WRITERESULT_OBJ_P(getThis());
 
+	zend_replace_error_handling(EH_THROW, phongo_exception_from_phongo_domain(PHONGO_ERROR_INVALID_ARGUMENT), &error_handling);
 	if (zend_parse_parameters_none() == FAILURE) {
+		zend_restore_error_handling(&error_handling);
 		return;
 	}
+	zend_restore_error_handling(&error_handling);
 
 	RETURN_BOOL(mongoc_write_concern_is_acknowledged(intern->write_concern));
 } /* }}} */
@@ -328,13 +372,15 @@ static void php_phongo_writeresult_free_object(zend_object* object) /* {{{ */
 	if (intern->write_concern) {
 		mongoc_write_concern_destroy(intern->write_concern);
 	}
+
+	if (!Z_ISUNDEF(intern->manager)) {
+		zval_ptr_dtor(&intern->manager);
+	}
 } /* }}} */
 
 static zend_object* php_phongo_writeresult_create_object(zend_class_entry* class_type) /* {{{ */
 {
-	php_phongo_writeresult_t* intern = NULL;
-
-	intern = PHONGO_ALLOC_OBJECT_T(php_phongo_writeresult_t, class_type);
+	php_phongo_writeresult_t* intern = zend_object_alloc(sizeof(php_phongo_writeresult_t), class_type);
 
 	zend_object_std_init(&intern->std, class_type);
 	object_properties_init(&intern->std, class_type);
@@ -344,13 +390,13 @@ static zend_object* php_phongo_writeresult_create_object(zend_class_entry* class
 	return &intern->std;
 } /* }}} */
 
-static HashTable* php_phongo_writeresult_get_debug_info(zval* object, int* is_temp) /* {{{ */
+static HashTable* php_phongo_writeresult_get_debug_info(phongo_compat_object_handler_type* object, int* is_temp) /* {{{ */
 {
 	php_phongo_writeresult_t* intern;
 	zval                      retval = ZVAL_STATIC_INIT;
 	bson_iter_t               iter;
 
-	intern   = Z_WRITERESULT_OBJ_P(object);
+	intern   = Z_OBJ_WRITERESULT(PHONGO_COMPAT_GET_OBJ(object));
 	*is_temp = 1;
 	array_init_size(&retval, 9);
 
@@ -431,11 +477,17 @@ void php_phongo_writeresult_init_ce(INIT_FUNC_ARGS) /* {{{ */
 	php_phongo_handler_writeresult.offset         = XtOffsetOf(php_phongo_writeresult_t, std);
 } /* }}} */
 
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
+php_phongo_writeresult_t* phongo_writeresult_init(zval* return_value, bson_t* reply, zval* manager, uint32_t server_id) /* {{{ */
+{
+	php_phongo_writeresult_t* writeresult;
+
+	object_init_ex(return_value, php_phongo_writeresult_ce);
+
+	writeresult            = Z_WRITERESULT_OBJ_P(return_value);
+	writeresult->reply     = bson_copy(reply);
+	writeresult->server_id = server_id;
+
+	ZVAL_ZVAL(&writeresult->manager, manager, 1, 0);
+
+	return writeresult;
+} /* }}} */
